@@ -1,10 +1,9 @@
-import numpy as np
 import dask.array as da
+import numpy as np
 from dask import delayed
 from omero.gateway import BlitzGateway
 from omero.model import enums as omero_enums
 
-
 # convert OMERO definitions into numpy types
 PIXEL_TYPES = {
     omero_enums.PixelsTypeint8: np.int8,
@@ -56,11 +55,13 @@ class Argo:
 
     # standard method required for Python's with statement
     def __exit__(self, *exc):
+        for e in exc:
+            if e is not None:
+                print(e)
+
         self.conn.close()
         return False
 
-###
-
 
 def get_data_lazy(image) -> da.Array:
     """
