@@ -3,7 +3,7 @@ import struct
 
 
 def clean_ascii(text):
-    return re.sub(r'[^\x20-\x7F]', '.', text)
+    return re.sub(r"[^\x20-\x7F]", ".", text)
 
 
 def xxd(x, start=0, stop=None):
@@ -18,25 +18,31 @@ def xxd(x, start=0, stop=None):
             if (r + 1) % 4 == 0:
                 print("  ", end="")
         # ASCII
-        print("   ", clean_ascii(x[i:i + 8].decode('utf-8', errors='ignore')),
-              "   ", end="")
+        print(
+            "   ",
+            clean_ascii(x[i : i + 8].decode("utf-8", errors="ignore")),
+            "   ",
+            end="",
+        )
         # Int32
-        print('{:>10} {:>10}'.format(*struct.unpack('II', x[i: i + 8])),
-              end="   ")
+        print(
+            "{:>10} {:>10}".format(*struct.unpack("II", x[i : i + 8])),
+            end="   ",
+        )
         print("")  # Newline
     return
 
 
 # Buffer reading functions
 def read_int(buffer, n=1):
-    res = struct.unpack('I' * n, buffer.read(4 * n))
+    res = struct.unpack("I" * n, buffer.read(4 * n))
     if n == 1:
         res = res[0]
     return res
 
 
 def read_string(buffer):
-    return ''.join([x.decode() for x in iter(lambda: buffer.read(1), b'\x00')])
+    return "".join([x.decode() for x in iter(lambda: buffer.read(1), b"\x00")])
 
 
 def read_delim(buffer, n):
