@@ -1,24 +1,24 @@
 import logging
 from time import perf_counter
+from typing import Callable, Dict, List
 
-from typing import Union, List, Dict, Callable
-
+import h5py
 import numpy as np
 import pandas as pd
-
-from extraction.core.functions.loaders import (
-    load_funs,
-    load_custom_args,
-    load_redfuns,
-    load_mergefuns,
-)
-from extraction.core.functions.defaults import exparams_from_meta
-from extraction.core.functions.distributors import trap_apply, reduce_z
-from extraction.core.functions.utils import depth
-from aliby.tile.tiler import Tiler
-from agora.abc import ProcessABC, ParametersABC
+from agora.abc import ParametersABC, ProcessABC
+from agora.io.cells import Cells
 from agora.io.writer import Writer, load_attributes
-from agora.io.cells import CellsLinear
+
+from aliby.tile.tiler import Tiler
+from extraction.core.functions.defaults import exparams_from_meta
+from extraction.core.functions.distributors import reduce_z, trap_apply
+from extraction.core.functions.loaders import (
+    load_custom_args,
+    load_funs,
+    load_mergefuns,
+    load_redfuns,
+)
+from extraction.core.functions.utils import depth
 
 CELL_FUNS, TRAPFUNS, FUNS = load_funs()
 CUSTOM_FUNS, CUSTOM_ARGS = load_custom_args()
@@ -142,11 +142,6 @@ class Extractor(ProcessABC):
             self._group = "/extraction/"
         return self._group
 
-    @property
-    def pos_file(self, store_name="store.h5"):
-        if not hasattr(self, "_pos_file"):
-            return self.local
-
     def load_custom_funs(self):
         """
         Load parameters of functions that require them from expt.
@@ -329,12 +324,33 @@ class Extractor(ProcessABC):
         labels=None,
         **kwargs,
     ) -> dict:
-        """
-        Extract individual time-point from images and masks
+        """Core extraction method for an individual time-point.
+
+        Parameters
+        ----------
+        tp : int
+            Time-point being analysed.
+        tree : dict
+            Nested dictionary indicating channels, reduction functions and
+            metrics to use during extraction.
+        tile_size : int
+            size of the tile to be extracted.
+        masks : np.ndarray
+            3-D boolean numpy array with dimensions (ncells, tile_size,
+            tile_size.
+        labels : t.List[t.List[int]]
+            List of list of ints indicating the ids of masks.
+        **kwargs : Additional keyword arguments to be passed to extractor.reduce_extract.
+
+        Returns
+        -------
+        dict
+
+        Examples
+        --------
+        FIXME: Add docs.
+
 
-        :param tp: int timepoint from which to extract results
-        :param tree: dict of dict {channel : {reduction_function : metrics}}
-        :**kwargs: Must include masks and preferably labels.
         """
 
         if tree is None:
@@ -343,7 +359,7 @@ class Extractor(ProcessABC):
         ch_tree = {ch: v for ch, v in tree.items() if ch != "general"}
         tree_chs = (*ch_tree,)
 
-        cells = CellsLinear(self.local)
+        cells = Cells(self.local)
 
         # labels
         if labels is None:
@@ -354,16 +370,8 @@ class Extractor(ProcessABC):
             }
 
         # masks
-        t = perf_counter()
         if masks is None:
             raw_masks = cells.at_time(tp, kind="mask")
-            nmasks = len([y.shape for x in raw_masks.values() for y in x])
-            # plt.imshow(np.dstack(raw_masks.get(1, [[]])).sum(axis=2))
-            # plt.savefig(f"{tp}.png")
-            # plt.close()
-            logging.debug(f"Timing:nmasks:{nmasks}")
-            logging.debug(f"Timing:MasksFetch:TP_{tp}:{perf_counter() - t}s")
-
             masks = {trap_id: [] for trap_id in range(cells.ntraps)}
             for trap_id, cells in raw_masks.items():
                 if len(cells):
@@ -372,7 +380,7 @@ class Extractor(ProcessABC):
         masks = [np.array(v) for v in masks.values()]
 
         # traps
-        traps = self.get_traps(tp, tile_size=tile_size, channels=tree_chs)
+        traps = self.get_traps(tp, tile_shape=tile_size, channels=tree_chs)
 
         self.img_bgsub = {}
         if self.params.sub_bg:
